@@ -1,3 +1,5 @@
+from typing import Dict
+
 from Bio import SeqIO
 
 from .base import Alignment
@@ -7,9 +9,13 @@ class Faidx(Alignment):
     def __init__(self, args) -> None:
         super().__init__(**self.process_args(args))
 
-    def run(self):
+    def run(self) -> None:
         record_dict = SeqIO.index(self.fasta, "fasta")
-        print(f">{record_dict[self.entry].name}\n{record_dict[self.entry].seq}")
 
-    def process_args(self, args):
+        # Split entries and iterate
+        for e in map(str.strip, self.entry.split(",")):
+            record = record_dict[e]
+            print(f">{record.name}\n{record.seq}")
+
+    def process_args(self, args) -> Dict[str, str]:
         return dict(fasta=args.fasta, entry=args.entry)
