@@ -1,3 +1,5 @@
+from typing import Dict
+
 from .base import Tree
 
 
@@ -5,18 +7,11 @@ class EvolutionaryRate(Tree):
     def __init__(self, args) -> None:
         super().__init__(**self.process_args(args))
 
-    def run(self):
+    def run(self) -> None:
         tree = self.read_tree_file()
-        total_tree_length = self.calculate_total_tree_length(tree)
+        total_tree_length = tree.total_branch_length()
         num_terminals = tree.count_terminals()
         print(round(total_tree_length / num_terminals, 4))
 
-    def process_args(self, args):
+    def process_args(self, args) -> Dict[str, str]:
         return dict(tree_file_path=args.tree)
-
-    def calculate_total_tree_length(self, tree):
-        total_len = float(0.0)
-        total_len = tree.total_branch_length()
-
-        if isinstance(total_len, (int, float)):
-            return total_len
