@@ -1,4 +1,6 @@
 import sys
+import copy
+from typing import Dict
 
 from .base import Tree
 
@@ -11,22 +13,20 @@ class LastCommonAncestorSubtree(Tree):
 
     def run(self):
         tree = self.read_tree_file()
+        # Make a deep copy to avoid issues with cached tree modifications
+        tree_copy = copy.deepcopy(tree)
         try:
             taxa = read_single_column_file_to_list(self.list_of_taxa)
         except FileNotFoundError:
             print("Taxa list file is not found. Please check pathing.")
-            sys.exit()
-        tree = tree.common_ancestor(taxa)
+            sys.exit(2)
+        subtree = tree_copy.common_ancestor(taxa)
 
-        self.write_tree_file(tree, self.output_file_path)
+        self.write_tree_file(subtree, self.output_file_path)
 
-    def process_args(self, args):
+    def process_args(self, args) -> Dict[str, str]:
         tree_file_path = args.tree
-
-        if args.output is None:
-            output_file_path = f"{tree_file_path}.subtree.tre"
-        else:
-            output_file_path = f"{args.output}"
+        output_file_path = args.output or f"{tree_file_path}.subtree.tre"
 
         return dict(
             tree_file_path=tree_file_path,
