@@ -1,3 +1,6 @@
+from typing import Dict
+import copy
+
 from Bio import Phylo
 
 from .base import Tree
@@ -10,18 +13,16 @@ class PrintTree(Tree):
     def run(self):
         tree = self.read_tree_file()
 
-        # remove branch lengths if specified by user
-        # otherwise, print ascii tree
         if self.remove:
-            for internode in tree.get_nonterminals():
-                internode.branch_length = None
-            for leaf in tree.get_terminals():
-                leaf.branch_length = None
+            # Make a deep copy to avoid modifying the cached tree
+            tree = copy.deepcopy(tree)
+            for node in tree.get_terminals() + tree.get_nonterminals():
+                node.branch_length = None
 
         try:
             Phylo.draw_ascii(tree)
         except BrokenPipeError:
             pass
 
-    def process_args(self, args):
+    def process_args(self, args) -> Dict[str, str]:
         return dict(tree_file_path=args.tree, remove=args.remove)
