@@ -1,3 +1,5 @@
+from typing import Dict
+
 from .base import Tree
 
 
@@ -5,13 +7,12 @@ class TipLabels(Tree):
     def __init__(self, args) -> None:
         super().__init__(**self.process_args(args))
 
-    def run(self):
+    def run(self) -> None:
         tree = self.read_tree_file()
         try:
-            for leaf in tree.get_terminals():
-                print(leaf.name)
+            print("\n".join([tip.name for tip in tree.get_terminals()]))
         except BrokenPipeError:
             pass
 
-    def process_args(self, args):
+    def process_args(self, args) -> Dict[str, str]:
         return dict(tree_file_path=args.tree)
