@@ -1,3 +1,6 @@
+from typing import Dict, Union
+
+from Bio.Phylo import Newick
 from .base import Tree
 
 
@@ -5,16 +8,17 @@ class TotalTreeLength(Tree):
     def __init__(self, args) -> None:
         super().__init__(**self.process_args(args))
 
-    def run(self):
+    def run(self) -> None:
         tree = self.read_tree_file()
-        total_tree_length = self.calculate_total_tree_length(tree)
-        print(round(total_tree_length, 4))
+        print(round(self.calculate_total_tree_length(tree), 4))
 
-    def process_args(self, args):
+    def process_args(self, args) -> Dict[str, str]:
         return dict(tree_file_path=args.tree)
 
-    def calculate_total_tree_length(self, tree):
-        total_len = float(0.0)
+    def calculate_total_tree_length(
+        self,
+        tree: Newick.Tree
+    ) -> Union[int, float]:
         total_len = tree.total_branch_length()
 
         if isinstance(total_len, (int, float)):
